package org.gant;

import android.app.Activity;
import android.os.Bundle;
import android.os.Handler;
import android.os.Message;
import android.widget.TextView;
import android.widget.Button;
import android.widget.Toast;
import android.view.View;
import android.bluetooth.BluetoothDevice;
import android.bluetooth.BluetoothAdapter;
import android.bluetooth.BluetoothSocket;
import java.util.Set;
import java.util.UUID;
import java.lang.Thread;
import java.io.IOException;
import java.io.InputStream;

public class MainActivity extends Activity {

	// Dclarations de variables
	
	private String message;

	// Constantes

	private static final int SUCCEEDED = 1, FAILED = 0, MESSAGE_READ = 2;
	private static final String DEVICE_NAME = "JY-MCU"; // Le nom du priphrique bluetooth
	private static final UUID MY_UUID = UUID.fromString("00001101-0000-1000-8000-00805F9B34FB"); // Sert  identifier l'application lors de la connexion bluetooth

	// Bluetooth

	private BluetoothAdapter bluetoothAdapter;
	private ConnectThread connectThread;
	private ConnectedThread connectedThread;

	// GUI

	private Button reconnectButton;
	private Button clearButton;
	private TextView messageView;
	private TextView connexionState;


	// Mthode appele au lancement de l'application

	@Override
	public void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		setContentView(R.layout.connecting);
		
		connexionState = (TextView)findViewById(R.id.connexionState);
		reconnectButton = (Button)findViewById(R.id.reconnectButton);
    	
		// On rcupre l'accs au bluetooth
		bluetoothAdapter = BluetoothAdapter.getDefaultAdapter();
		if(!bluetoothAdapter.isEnabled()) { // On l'active si ce n'est pas dj fait
			bluetoothAdapter.enable();
			while(!bluetoothAdapter.isEnabled()) { }
		}

		launchConnexion();
	}

	public void launchConnexion() {
		Set<BluetoothDevice> pairedDevices = bluetoothAdapter.getBondedDevices(); // On rcupre la liste des priphriques bluetooth
		for(BluetoothDevice device : pairedDevices) { // On cherche s'il y en a un qui correspond à celui qu'on cherche (DEVICE_NAME)
			if(device.getName().equals(DEVICE_NAME)) {
				connexionState.setText("Connexion en cours..."); // On affiche la tentative de connexion
				connectThread = new ConnectThread(device); // On cre un thread de connexion
				connectThread.start(); // On le lance
			}
		}
	}
	


	// Afficher la vue main

	public void displayMain() {
		setContentView(R.layout.main);
		messageView = (TextView)findViewById(R.id.messageView);
		message = " ";
	}

	// Permet de grer la connexion

	private Handler connexionHandler = new Handler() {
		@Override
		public void handleMessage(Message msg) {
			switch(msg.what) {
			case SUCCEEDED : // Si a russit
				displayMain();
				break;
			case FAILED : // Si a choue
				connexionState.setText("Connexion choue"); // On l'indique
				reconnectButton.setVisibility(View.VISIBLE);
				break;
			case MESSAGE_READ : // Si un message est reu
				byte[] buffer = (byte[])msg.obj;
				String messageRead = ""; //Integer.toString((int)buffer[0]); //= new String(buffer, 0, msg.arg1);
				for(int i = 0; i < buffer.length; ++i)
				{
					char c = (char)buffer[i];
					if(c > 31 && c < 128)
						messageRead += c;
				}
				message = message.concat(messageRead);
				messageView.setText(message);
				break;
			}
		}
	};

	// Callback du bouton reconnectButton

	public void reconnect(View view) {
		reconnectButton.setVisibility(View.GONE);
		connexionState.setText("Connexion en cours...");
		launchConnexion();
	}

	// Callback du bouton clearButton

	public void clear(View view) {
		message = " ";
		messageView.setText(message);
	}

	// Thread de connexion
	
	private class ConnectThread extends Thread { // Thread de connexion
		private final BluetoothSocket mmSocket;
		private final BluetoothDevice mmDevice;

		public ConnectThread(BluetoothDevice device) {
			BluetoothSocket tmp = null;
			mmDevice = device;

			try {
				tmp = mmDevice.createRfcommSocketToServiceRecord(MY_UUID); // On cre le socket bluetooth
			} catch (IOException e) { }

			mmSocket = tmp;
		}

		public void run() { // Au lancement du thread
			try {
				mmSocket.connect(); // On lance la connexion
			} catch (IOException connectException) { // En cas d'erreur
				try {
					mmSocket.close(); // On ferme le socket
				} catch (IOException closeException) { }
				connexionHandler.sendMessage(connexionHandler.obtainMessage(FAILED)); // On envoie le message d'erreur	
				return;
			}
			connectedThread = new ConnectedThread(mmSocket); // On cre un thread de controle avec le socket bluetooth
			connectedThread.start(); // On le lance
			connexionHandler.sendMessage(connexionHandler.obtainMessage(SUCCEEDED)); // On envoie le message de russite
		}

		public void cancel() { //  la fermeture de thread
			try {
				mmSocket.close(); // On ferme le socket
			} catch (IOException e) { }
		}
	}

	// Thread de gestion de la connexion

	private class ConnectedThread extends Thread { // Thread de controle
		private final BluetoothSocket mmSocket;
		private final InputStream mmInStream;

		public ConnectedThread(BluetoothSocket socket) {
			mmSocket = socket;
			InputStream tmpIn = null;

			try {
				tmpIn = socket.getInputStream(); // Ouvre un flux entrant sur le socket pass en paramtre
			} catch (IOException e) { }

			mmInStream = tmpIn;
		}

		public void run() {
			byte[] buffer = new byte[1024];  // buffer store for the stream
			int bytes; // bytes returned from read()
		 
			while (true) {
				try {
					bytes = mmInStream.read(buffer);
					connexionHandler.sendMessage(connexionHandler.obtainMessage(MESSAGE_READ, bytes, -1, buffer));
				} catch (IOException e) {
					break;
				}
			}
		}

		public void cancel() { //  la fin du thread
			try {
				mmSocket.close(); // On ferme le socket
			} catch (IOException e) { }
		}
	}
}